from Bangla_translator import bangla_google_translator
trans=bangla_google_translator()
print(trans.translate_B2E('কারাগারে থাকা ১৪ জনের মধ্যে ১২ জন স্বীকারোক্তিমূলক জবানবন্দী দিয়েছে'))
# 12 of the 14 people in prison gave confessional statements
print(trans.translate_E2B('12 of the 14 people in prison gave confessional statements'))
# কারাগারে 14 জনের মধ্যে 1২ জন স্বীকারোক্তিমূলক বক্তব্য দিয়েছেন